#ifndef REQUESTMANAGER_HH
#define REQUESTMANAGER_HH 1

#include "SliceData.hh"
#include "SliceSAX2Handler.hh"
#include "ConsensusRequest.hh"
#include "Foundation.hh"
#include <string>

using libSlice::SliceData;
using libSlice::ReadManager;
using libSlice::ReadData;

using namespace std;


class SafeException
{
public:
  SafeException() {};
  ~SafeException() {};
};


class RequestManager
{
public:

  RequestManager(const string & logCategory);
  virtual ~RequestManager();

  virtual void addSimpleOption(const char * name, const char * value);

  virtual void addRequest(ConsensusRequest * requestData);
  virtual void addReadManager(ReadManager * readManager);
  virtual void addReadData(ReadData * readData);

  virtual void addSliceRange(long & index, long & gindex);
  virtual void addSlice(SliceData * slice);

  virtual int  printResults();
  virtual void printStats(ostream & out, const string & prefix);

  virtual void errorSliceHandler(SliceData * slice, const Exception & e);

  void setDoStats             (int doStats);
  void setDoAmbiguity         (int doAmbiguity);
  void setDoConsensus         (int doConsensus);
  void setScoringModel        (int scoringModel);

  void setHighQualityThreshold(int highQualityThreshold);

  int processFile(const string & infile, ostream & error);

  static ofstream * openOutputFile(const string & filename);

  static string     getVersionString(char * gVersionStr, 
                                     char * gRevisionStr);
  static string     getOptionHelp();


  static void handleOptions(Foundation * tf);
  static void addStandardOptions(Foundation * tf);
  static void handleOptionResult();

  static void setDoAmbiguity         (const string & value);
  static void setDoConsensus         (const string & value);
  static void setScoringModel        (const string & value);
  static void setHighQualityThreshold(const string & value);

  static void print60(ostream & out, const string & bases);

  ostream & getOutputStream() const;

protected:

  typedef void (*OptionHandler)(const string &);
  typedef map<string, OptionHandler> OptionMap;

  OptionMap m_optionMap;

  void registerOptionHandler(const string & name, OptionHandler callback);

  void printXMLHeader(ConsensusRequest * requestData=NULL, 
                      ostream * outstream = NULL);

  ConsensusRequest * m_requestData;
  ReadManager * m_readManager;

  ostream & m_out;

  LogCategory m_log;

  bool m_doLogging;

  string m_infile;

  // Static Members
  static Foundation * m_tf;
  static string m_outfile;

  static int  m_highQualityThreshold;
  static int  m_errorCount;

  static int  m_recallEmpty;
  static int  m_doStats;
  static int  m_doAmbiguity;
  static int  m_doConsensus;
  static int  m_scoringModel;
  static ofstream * m_outfileStream;
};

#endif
