#include "ReadMerger.hh"
#include <cstdio>

ReadMerger::ReadMerger() : RequestManager("ReadMerger")
{
  m_readCount = 0;

  m_log.debugL("Contructed ReadMerger");
  m_log.setMessageLevel(MESSAGE_LEVEL_DEBUG_MEDIUM);
}

ReadMerger::~ReadMerger()
{
  m_log.debugL("Destructed ReadMerger");
}



void ReadMerger::addSliceRange(long & index, long & gindex)
{
  // Don't bother with the slices, break the processing
  throw SafeException();
}

/*
void ReadMerger::addSlice(SliceData * slice)
{
  delete slice;
}
*/

void ReadMerger::addReadData(ReadData * readData)
{
  const string & seqname = readData->getSeqname();
  string seqnum = lookupSeqId(seqname);

  if (seqnum == "")
  {
    char buffer [16];
    snprintf(buffer, sizeof(buffer), "%d", m_readCount);
    m_readCount++;

    readData->resetId(buffer);
    readData->printXML(m_out);
    
    m_readMap[seqname] = (string) buffer;
  }
  else
  {
    m_log.pushMessageLevel(MESSAGE_LEVEL_DEBUG_LOW);
    m_log << "Duplicate seq(" << seqnum << "):" << seqname << endl;
    m_log.popMessageLevel();
  }
}

void ReadMerger::addRequest(ConsensusRequest * request)
{
  m_log.pushMessageLevel(MESSAGE_LEVEL_DEBUG_LOW);
  m_log << "parsing: " << m_infile << endl;
  m_log.popMessageLevel();

  m_fileList.push_back(m_infile);
  m_fileListIter = m_fileList.begin();

  if (!m_requestData)
  {
    m_requestData = request;
    printXMLHeader(request);

    m_out << "  <ReadCollection>" << endl;
  }
  else
  {
    delete request;
  }
}

string ReadMerger::getNextFile()
{
  string retval = "";

  if (m_fileListIter != m_fileList.end())
  {
    retval = *m_fileListIter;
    m_fileListIter++;
  }

  return retval;
}

string ReadMerger::lookupSeqId(const string & seqname)
{
  string retval = "";

  ReadMap::const_iterator i = m_readMap.find(seqname);
  if (i != m_readMap.end())
  {
    retval = i->second;
  }

  return retval;
}
