#include "SliceMerger.hh"
#include <cstdio>

SliceMerger::SliceMerger(ReadMerger & readMerger)
  : RequestManager("SliceMerger"),
    m_readMerger(readMerger)
{
  m_firstSlice = true;
  m_newIndex = m_newGindex = 0L;

  m_log.debugL("Contructed SliceMerger");
  m_log.setMessageLevel(MESSAGE_LEVEL_DEBUG_MEDIUM);
}

string SliceMerger::s_slicelist;

SliceMerger::~SliceMerger()
{
  m_log.debugL("Destructed SliceMerger");
}

void SliceMerger::addSliceRange(long & index, long & gindex)
{
  if (m_firstSlice)
  {
    m_out << "  </ReadCollection>"    << endl
          << "  <SliceRange Index=\"" << index
          << "\" Gindex=\""           << gindex
          << "\">" << endl;
  }

  index = gindex = 0L;
}

void SliceMerger::addSlice(SliceData * slice)
{
  if (m_firstSlice)
  {
    m_newIndex = slice->getIndex();
    m_newGindex = slice->getGindex();

    m_firstSlice = false;
  }

  RequestManager::addSlice(slice);

  char consensus = slice->getConsensus();

  long coordinate = m_newIndex;
  if (coordinate && consensus == '-') coordinate--;

  slice->resetIndex(coordinate, m_newGindex);

  // Renumber the read ids
  string newid = "";

  unsigned short depth = slice->getDepth();

  for (unsigned short i = 0; i < depth; i++)
  {
    if (i)
    {
      newid += " ";
    }

    const string & seqname = slice->getRead(i)->getSeqname();
    newid += m_readMerger.lookupSeqId(seqname);
  }

  slice->resetReadId();
  slice->addReadId(newid.c_str());
  slice->printXML(m_out);

  if (consensus != '-') m_newIndex++;
  m_newGindex++;
}

void SliceMerger::endSlices()
{
  m_out << "  </SliceRange>" << endl;
  m_out << "</Request>"      << endl;
}

void SliceMerger::handleOptions(Foundation * tf)
{
  tf->getOptions()->addOptionResult("A|slicelist=s", &s_slicelist,
                                    "Specify file listing slice files to merge");

  RequestManager::handleOptions(tf);
}


void SliceMerger::addSliceList(ReadMerger::StringList & inputfiles)
{
  if (!s_slicelist.empty())
  {
    ifstream * slicelistfile = FileSystem::openInputFile(s_slicelist);
    string buffer;

    while (*slicelistfile)
    {
      *slicelistfile >> buffer;

      if (*slicelistfile)
      {
        inputfiles.push_back(buffer);
      }
    }
  }
}
